@@ -20,17 +20,23 @@ from __future__ import print_function
 
 import numpy as np
 from scipy import stats
-import tensorflow as tf
+from tensorflow.contrib import distributions as distributions_lib
+from tensorflow.python.framework import constant_op
+from tensorflow.python.framework import dtypes
+from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import math_ops
+from tensorflow.python.ops import nn_ops
+from tensorflow.python.platform import test
 
-distributions = tf.contrib.distributions
+distributions = distributions_lib
 
 
-class MultivariateNormalShapeTest(tf.test.TestCase):
+class MultivariateNormalShapeTest(test.TestCase):
 
   def _testPDFShapes(self, mvn_dist, mu, sigma):
     with self.test_session() as sess:
       mvn = mvn_dist(mu, sigma)
-      x = 2 * tf.ones_like(mu)
+      x = 2 * array_ops.ones_like(mu)
 
       log_pdf = mvn.log_pdf(x)
       pdf = mvn.pdf(x)
@@ -41,8 +47,8 @@ class MultivariateNormalShapeTest(tf.test.TestCase):
       x_value = 2. * np.ones([3, 3, 2])
       feed_dict = {mu: mu_value, sigma: sigma_value}
 
-      scipy_mvn = stats.multivariate_normal(mean=mu_value[(0, 0)],
-                                            cov=sigma_value[(0, 0)])
+      scipy_mvn = stats.multivariate_normal(
+          mean=mu_value[(0, 0)], cov=sigma_value[(0, 0)])
       expected_log_pdf = scipy_mvn.logpdf(x_value[(0, 0)])
       expected_pdf = scipy_mvn.pdf(x_value[(0, 0)])
 
@@ -53,19 +59,19 @@ class MultivariateNormalShapeTest(tf.test.TestCase):
       self.assertAllClose(expected_pdf, pdf_evaled[0, 0])
 
   def testPDFUnknownSize(self):
-    mu = tf.placeholder(tf.float32, shape=(3 * [None]))
-    sigma = tf.placeholder(tf.float32, shape=(4 * [None]))
+    mu = array_ops.placeholder(dtypes.float32, shape=(3 * [None]))
+    sigma = array_ops.placeholder(dtypes.float32, shape=(4 * [None]))
     self._testPDFShapes(distributions.MultivariateNormalFull, mu, sigma)
     self._testPDFShapes(distributions.MultivariateNormalCholesky, mu, sigma)
 
   def testPDFUnknownShape(self):
-    mu = tf.placeholder(tf.float32)
-    sigma = tf.placeholder(tf.float32)
+    mu = array_ops.placeholder(dtypes.float32)
+    sigma = array_ops.placeholder(dtypes.float32)
     self._testPDFShapes(distributions.MultivariateNormalFull, mu, sigma)
     self._testPDFShapes(distributions.MultivariateNormalCholesky, mu, sigma)
 
 
-class MultivariateNormalDiagTest(tf.test.TestCase):
+class MultivariateNormalDiagTest(test.TestCase):
   """Well tested because this is a simple override of the base class."""
 
   def setUp(self):
@@ -99,8 +105,8 @@ class MultivariateNormalDiagTest(tf.test.TestCase):
     diag_v = [[1.0, 5.0]]
 
     with self.test_session():
-      mu_ph = tf.placeholder(tf.float32, name="mu_ph")
-      diag_ph = tf.placeholder(tf.float32, name="diag_ph")
+      mu_ph = array_ops.placeholder(dtypes.float32, name="mu_ph")
+      diag_ph = array_ops.placeholder(dtypes.float32, name="diag_ph")
       dist = distributions.MultivariateNormalDiag(
           mu_ph, diag_ph, validate_args=True)
       with self.assertRaisesOpError("mu should have rank"):
@@ -112,7 +118,7 @@ class MultivariateNormalDiagTest(tf.test.TestCase):
     with self.test_session():
       dist = distributions.MultivariateNormalDiag(mu, diag)
       samps = dist.sample(1000, seed=0).eval()
-      cov_mat = tf.matrix_diag(diag).eval()**2
+      cov_mat = array_ops.matrix_diag(diag).eval()**2
 
       self.assertAllClose(mu, samps.mean(axis=0), atol=0.1)
       self.assertAllClose(cov_mat, np.cov(samps.T), atol=0.1)
@@ -123,13 +129,13 @@ class MultivariateNormalDiagTest(tf.test.TestCase):
     with self.test_session():
       dist = distributions.MultivariateNormalDiagWithSoftplusStDev(mu, diag)
       samps = dist.sample(1000, seed=0).eval()
-      cov_mat = tf.matrix_diag(tf.nn.softplus(diag)).eval()**2
+      cov_mat = array_ops.matrix_diag(nn_ops.softplus(diag)).eval()**2
 
       self.assertAllClose(mu, samps.mean(axis=0), atol=0.1)
       self.assertAllClose(cov_mat, np.cov(samps.T), atol=0.1)
 
 
-class MultivariateNormalDiagPlusVDVTTest(tf.test.TestCase):
+class MultivariateNormalDiagPlusVDVTTest(test.TestCase):
   """Well tested because this is a simple override of the base class."""
 
   def setUp(self):
@@ -152,8 +158,7 @@ class MultivariateNormalDiagPlusVDVTTest(tf.test.TestCase):
     v = self._rng.rand(3, 2)  # v works with diag_large.
     with self.test_session():
       with self.assertRaisesRegexp(ValueError, "shape.*should match"):
-        distributions.MultivariateNormalDiagPlusVDVT(
-            mu, diag_large, v)
+        distributions.MultivariateNormalDiagPlusVDVT(mu, diag_large, v)
 
   def testNonmatchingMuDiagDimensionsFailsDynamic(self):
     mu = self._rng.rand(2)
@@ -162,9 +167,9 @@ class MultivariateNormalDiagPlusVDVTTest(tf.test.TestCase):
     v = self._rng.rand(3, 2)  # v works with diag_large.
 
     with self.test_session():
-      mu_ph = tf.placeholder(tf.float32, name="mu_ph")
-      v_ph = tf.placeholder(tf.float32, name="v_ph")
-      diag_ph = tf.placeholder(tf.float32, name="diag_ph")
+      mu_ph = array_ops.placeholder(dtypes.float32, name="mu_ph")
+      v_ph = array_ops.placeholder(dtypes.float32, name="v_ph")
+      diag_ph = array_ops.placeholder(dtypes.float32, name="diag_ph")
       dist = distributions.MultivariateNormalDiagPlusVDVT(
           mu_ph, diag_ph, v_ph, validate_args=True)
       with self.assertRaisesOpError("mu.*cov.*shape"):
@@ -184,16 +189,16 @@ class MultivariateNormalDiagPlusVDVTTest(tf.test.TestCase):
       self.assertAllClose(cov_mat, np.cov(samps.T), atol=0.1)
 
 
-class MultivariateNormalCholeskyTest(tf.test.TestCase):
+class MultivariateNormalCholeskyTest(test.TestCase):
 
   def setUp(self):
     self._rng = np.random.RandomState(42)
 
   def _random_chol(self, *shape):
     mat = self._rng.rand(*shape)
-    chol = distributions.matrix_diag_transform(mat, transform=tf.nn.softplus)
-    chol = tf.matrix_band_part(chol, -1, 0)
-    sigma = tf.matmul(chol, chol, adjoint_b=True)
+    chol = distributions.matrix_diag_transform(mat, transform=nn_ops.softplus)
+    chol = array_ops.matrix_band_part(chol, -1, 0)
+    sigma = math_ops.matmul(chol, chol, adjoint_b=True)
     return chol.eval(), sigma.eval()
 
   def testNonmatchingMuSigmaFailsStatic(self):
@@ -210,8 +215,8 @@ class MultivariateNormalCholeskyTest(tf.test.TestCase):
 
   def testNonmatchingMuSigmaFailsDynamic(self):
     with self.test_session():
-      mu_ph = tf.placeholder(tf.float64)
-      chol_ph = tf.placeholder(tf.float64)
+      mu_ph = array_ops.placeholder(dtypes.float64)
+      chol_ph = array_ops.placeholder(dtypes.float64)
 
       mu_v = self._rng.rand(2)
       chol_v, _ = self._random_chol(2, 2, 2)
@@ -316,7 +321,7 @@ class MultivariateNormalCholeskyTest(tf.test.TestCase):
       mu = self._rng.rand(2)
       chol, sigma = self._random_chol(2, 2)
 
-      n = tf.constant(100000)
+      n = constant_op.constant(100000)
       mvn = distributions.MultivariateNormalCholesky(mu, chol)
       samples = mvn.sample(n, seed=137)
       sample_values = samples.eval()
@@ -338,8 +343,7 @@ class MultivariateNormalCholeskyTest(tf.test.TestCase):
       # Check sample means
       x = samples_val[:, :, :, 1, 1, :]
       self.assertAllClose(
-          x.reshape(10 * 11 * 12, 2).mean(axis=0),
-          mu[1, 1], atol=1e-2)
+          x.reshape(10 * 11 * 12, 2).mean(axis=0), mu[1, 1], atol=1e-2)
 
       # Check that log_prob(samples) works
       log_prob_val = mvn.log_prob(samples_val).eval()
@@ -354,17 +358,17 @@ class MultivariateNormalCholeskyTest(tf.test.TestCase):
       chol, sigma = self._random_chol(3, 5, 2, 2)
 
       mvn = distributions.MultivariateNormalCholesky(mu, chol)
-      n = tf.constant(100000)
+      n = constant_op.constant(100000)
       samples = mvn.sample(n, seed=137)
       sample_values = samples.eval()
 
       self.assertEqual(samples.get_shape(), (100000, 3, 5, 2))
       self.assertAllClose(
-          sample_values[:, 1, 1, :].mean(axis=0),
-          mu[1, 1, :], atol=0.05)
+          sample_values[:, 1, 1, :].mean(axis=0), mu[1, 1, :], atol=0.05)
       self.assertAllClose(
           np.cov(sample_values[:, 1, 1, :], rowvar=0),
-          sigma[1, 1, :, :], atol=1e-1)
+          sigma[1, 1, :, :],
+          atol=1e-1)
 
   def testShapes(self):
     with self.test_session():
@@ -382,7 +386,7 @@ class MultivariateNormalCholeskyTest(tf.test.TestCase):
       self.assertEqual((3, 5), tuple(mvn.batch_shape().eval()))
 
 
-class MultivariateNormalFullTest(tf.test.TestCase):
+class MultivariateNormalFullTest(test.TestCase):
 
   def setUp(self):
     self._rng = np.random.RandomState(42)
@@ -391,7 +395,7 @@ class MultivariateNormalFullTest(tf.test.TestCase):
     # This ensures sigma is positive def.
     mat_shape = batch_shape + event_shape + event_shape
     mat = self._rng.randn(*mat_shape)
-    sigma = tf.matmul(mat, mat, adjoint_b=True).eval()
+    sigma = math_ops.matmul(mat, mat, adjoint_b=True).eval()
 
     mu_shape = batch_shape + event_shape
     mu = self._rng.randn(*mu_shape)
@@ -427,10 +431,10 @@ class MultivariateNormalFullTest(tf.test.TestCase):
       self.assertEqual(batch_shape, kl.get_shape())
 
       kl_v = kl.eval()
-      expected_kl_0 = _compute_non_batch_kl(
-          mu_a[0, :], sigma_a[0, :, :], mu_b[0, :], sigma_b[0, :])
-      expected_kl_1 = _compute_non_batch_kl(
-          mu_a[1, :], sigma_a[1, :, :], mu_b[1, :], sigma_b[1, :])
+      expected_kl_0 = _compute_non_batch_kl(mu_a[0, :], sigma_a[0, :, :],
+                                            mu_b[0, :], sigma_b[0, :])
+      expected_kl_1 = _compute_non_batch_kl(mu_a[1, :], sigma_a[1, :, :],
+                                            mu_b[1, :], sigma_b[1, :])
       self.assertAllClose(expected_kl_0, kl_v[0])
       self.assertAllClose(expected_kl_1, kl_v[1])
 
@@ -465,4 +469,4 @@ def _compute_non_batch_kl(mu_a, sigma_a, mu_b, sigma_b):
 
 
 if __name__ == "__main__":
-  tf.test.main()
+  test.main()
