@@ -12,8 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
 """Tests for tensorflow.ops.gradients."""
+
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -21,7 +21,6 @@ from __future__ import print_function
 import warnings
 
 import numpy as np
-import tensorflow as tf
 
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
@@ -32,16 +31,19 @@ from tensorflow.python.framework import test_util
 from tensorflow.python.framework.constant_op import constant
 from tensorflow.python.ops import array_grad  # pylint: disable=unused-import
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import control_flow_grad  # pylint: disable=unused-import
+from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops import data_flow_grad  # pylint: disable=unused-import
 from tensorflow.python.ops import data_flow_ops  # pylint: disable=unused-import
+from tensorflow.python.ops import functional_ops  # pylint: disable=unused-import
 from tensorflow.python.ops import gradients
 from tensorflow.python.ops import gradients_impl
 from tensorflow.python.ops import math_grad  # pylint: disable=unused-import
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import nn_grad  # pylint: disable=unused-import
 from tensorflow.python.ops import state_grad  # pylint: disable=unused-import
-from tensorflow.python.ops import functional_ops  # pylint: disable=unused-import
-
+from tensorflow.python.ops import tensor_array_grad  # pylint: disable=unused-import
+from tensorflow.python.ops import tensor_array_ops
 from tensorflow.python.ops.nn_ops import bias_add
 from tensorflow.python.platform import googletest
 
@@ -86,22 +88,21 @@ class GradientsTest(test_util.TensorFlowTestCase):
     with ops.Graph().as_default() as g:
       t1 = constant(1.0)
       t2 = constant(2.0)
-      t3 = array_ops.pack([t1, t2])
+      t3 = array_ops.stack([t1, t2])
     # Full graph
     self._assertOpListEqual([t3.op, t2.op, t1.op],
                             _OpsBetween(g, [t3.op], [t1.op, t2.op]))
     # Only t1, t3.
-    self._assertOpListEqual([t3.op, t1.op],
-                            _OpsBetween(g, [t3.op], [t1.op]))
+    self._assertOpListEqual([t3.op, t1.op], _OpsBetween(g, [t3.op], [t1.op]))
 
   def testOpsBetweenUnreachable(self):
     with ops.Graph().as_default() as g:
       t1 = constant(1.0)
       t2 = constant(2.0)
-      _ = array_ops.pack([t1, t2])
+      _ = array_ops.stack([t1, t2])
       t4 = constant(1.0)
       t5 = constant(2.0)
-      t6 = array_ops.pack([t4, t5])
+      t6 = array_ops.stack([t4, t5])
     # Elements of to_ops are always listed.
     self._assertOpListEqual([t6.op], _OpsBetween(g, [t6.op], [t1.op]))
 
@@ -109,11 +110,11 @@ class GradientsTest(test_util.TensorFlowTestCase):
     with ops.Graph().as_default() as g:
       t1 = constant(1.0)
       t2 = constant(2.0)
-      t3 = array_ops.pack([t1, t2])
+      t3 = array_ops.stack([t1, t2])
       t4 = constant([1.0])
-      t5 = array_ops.concat(0, [t4, t3])
+      t5 = array_ops.concat([t4, t3], 0)
       t6 = constant([2.0])
-      t7 = array_ops.concat(0, [t5, t6])
+      t7 = array_ops.concat([t5, t6], 0)
     self._assertOpListEqual([t7.op, t5.op, t4.op],
                             _OpsBetween(g, [t7.op], [t4.op]))
 
@@ -121,11 +122,11 @@ class GradientsTest(test_util.TensorFlowTestCase):
     with ops.Graph().as_default() as g:
       t1 = constant(1.0)
       t2 = constant(2.0)
-      t3 = array_ops.pack([t1, t2])
-      t4 = array_ops.concat(0, [t3, t3, t3])
+      t3 = array_ops.stack([t1, t2])
+      t4 = array_ops.concat([t3, t3, t3], 0)
       t5 = constant([1.0])
-      t6 = array_ops.concat(0, [t4, t5])
-      t7 = array_ops.concat(0, [t6, t3])
+      t6 = array_ops.concat([t4, t5], 0)
+      t7 = array_ops.concat([t6, t3], 0)
     self._assertOpListEqual([t6.op, t4.op, t3.op],
                             _OpsBetween(g, [t6.op], [t3.op]))
     self._assertOpListEqual([t7.op, t6.op, t5.op, t4.op, t3.op, t1.op],
@@ -151,7 +152,7 @@ class GradientsTest(test_util.TensorFlowTestCase):
       w = constant(1.0, shape=[2, 2])
       x = constant(1.0, shape=[2, 2])
       wx = math_ops.matmul(w, x)
-      split_wx = array_ops.split(0, 2, wx)
+      split_wx = array_ops.split(value=wx, num_or_size_splits=2, axis=0)
       c = math_ops.reduce_sum(split_wx[1])
       gw = gradients.gradients(c, [w])[0]
     self.assertEquals("MatMul", gw.op.type)
@@ -205,7 +206,7 @@ class GradientsTest(test_util.TensorFlowTestCase):
     # Test that we don't differentiate 'x'. The gradient function for 'x' is
     # set explicitly to None so we will get an exception if the gradient code
     # tries to differentiate 'x'.
-    with ops.Graph().as_default() as g:
+    with ops.Graph().as_default():
       c = constant(1.0)
       x = array_ops.identity(c)
       y = x + 1.0
@@ -230,10 +231,9 @@ class GradientsTest(test_util.TensorFlowTestCase):
       y = x * 2.0
       z = y + y + y + y + y + y + y + y + y + y
       grads = gradients.gradients(
-          z,
-          [x, y],
-          aggregation_method=
-          gradients.AggregationMethod.EXPERIMENTAL_ACCUMULATE_N)
+          z, [x, y],
+          aggregation_method=gradients.AggregationMethod.
+          EXPERIMENTAL_ACCUMULATE_N)
       self.assertTrue(all(x is not None for x in grads))
       self.assertEqual(20.0, grads[0].eval())
       self.assertEqual(10.0, grads[1].eval())
@@ -244,9 +244,7 @@ class GradientsTest(test_util.TensorFlowTestCase):
       y = x * 2.0
       z = y + y + y + y + y + y + y + y + y + y
       grads = gradients.gradients(
-          z,
-          [x, y],
-          aggregation_method=gradients.AggregationMethod.ADD_N)
+          z, [x, y], aggregation_method=gradients.AggregationMethod.ADD_N)
       self.assertTrue(all(x is not None for x in grads))
       self.assertEqual(20.0, grads[0].eval())
       self.assertEqual(10.0, grads[1].eval())
@@ -257,8 +255,7 @@ class GradientsTest(test_util.TensorFlowTestCase):
       y = x * 2.0
       z = y + y + y + y + y + y + y + y + y + y
       grads = gradients.gradients(
-          z,
-          [x, y],
+          z, [x, y],
           aggregation_method=gradients.AggregationMethod.EXPERIMENTAL_TREE)
       self.assertTrue(all(x is not None for x in grads))
       self.assertEqual(20.0, grads[0].eval())
@@ -266,11 +263,13 @@ class GradientsTest(test_util.TensorFlowTestCase):
 
   def testNoGradientForStringOutputs(self):
     with ops.Graph().as_default():
+
       def _TestOpGrad(_, float_grad, string_grad):
         """Gradient function for TestStringOutput."""
         self.assertEquals(float_grad.dtype, dtypes.float32)
         self.assertFalse(string_grad)
         return float_grad
+
       ops.RegisterGradient("TestStringOutput")(_TestOpGrad)
 
       c = constant(1.0)
@@ -284,16 +283,34 @@ class GradientsTest(test_util.TensorFlowTestCase):
 
   def testSingletonIndexedSlices(self):
     with ops.Graph().as_default():
-      x = tf.placeholder(tf.float32)
-      y = tf.identity(x)
-      dy = tf.IndexedSlices(tf.placeholder(tf.float32),
-                            tf.placeholder(tf.int32))
+      x = array_ops.placeholder(dtypes.float32)
+      y = array_ops.identity(x)
+      dy = ops.IndexedSlices(
+          array_ops.placeholder(dtypes.float32),
+          array_ops.placeholder(dtypes.int32))
       dx, = gradients.gradients(y, x, grad_ys=dy)
       # The gradient of tf.identity should pass the value through unchanged.
       # A previous version of the code did this only for tf.Tensor, not
       # tf.IndexedSlices.
       self.assertEqual(dx, dy)
 
+  def testNonDifferentiableSwitchInWhileLoop(self):
+    with ops.Graph().as_default():
+      v = array_ops.placeholder(dtypes.float32, [])
+
+      def _Step(i, a, ta):
+        a += math_ops.cast(v, dtypes.int32)
+        return (i + 1, a, ta.write(i, a))
+
+      n = 4
+      i, _, ta = control_flow_ops.while_loop(
+          lambda i, *_: i < n,
+          _Step, [0, 0, tensor_array_ops.TensorArray(
+              dtypes.int32, size=n)])
+      target = ta.read(i - 1)
+      grad, = gradients.gradients(target, v)
+      self.assertIsNone(grad)
+
 
 class FunctionGradientsTest(test_util.TensorFlowTestCase):
 
@@ -315,12 +332,12 @@ class FunctionGradientsTest(test_util.TensorFlowTestCase):
 
   @classmethod
   def _GetFunc(cls, **kwargs):
-    return function.Defun(tf.float32, tf.float32, **kwargs)(
-        cls.XSquarePlusB)
+    return function.Defun(dtypes.float32, dtypes.float32, **
+                          kwargs)(cls.XSquarePlusB)
 
   def _GetFuncGradients(self, f, x_value, b_value):
-    x = tf.constant(x_value, name="x")
-    b = tf.constant(b_value, name="b")
+    x = constant_op.constant(x_value, name="x")
+    b = constant_op.constant(b_value, name="b")
 
     y = f(x, b)
     grads = gradients.gradients(y, [x, b])
@@ -339,9 +356,9 @@ class FunctionGradientsTest(test_util.TensorFlowTestCase):
   def testFunctionGradientsComposition(self):
     with ops.Graph().as_default():
       f = self._GetFunc()
-      x = tf.constant([2.0], name="x")
-      b1 = tf.constant([1.0], name="b1")
-      b2 = tf.constant([1.0], name="b2")
+      x = constant_op.constant([2.0], name="x")
+      b1 = constant_op.constant([1.0], name="b1")
+      b2 = constant_op.constant([1.0], name="b2")
 
       y = f(f(x, b1), b2)
       # Build gradient graph (should add SymbolicGradient node for function).
@@ -354,8 +371,8 @@ class FunctionGradientsTest(test_util.TensorFlowTestCase):
   def testFunctionGradientsWithGradFunc(self):
     g = ops.Graph()
     with g.as_default():
-      grad_func = function.Defun(tf.float32, tf.float32, tf.float32)(
-          self.XSquarePlusBGradient)
+      grad_func = function.Defun(dtypes.float32, dtypes.float32,
+                                 dtypes.float32)(self.XSquarePlusBGradient)
       f = self._GetFunc(grad_func=grad_func)
       # Get gradients (should add SymbolicGradient node for function, which
       # uses the grad_func above, which multiplies all gradients by 2).
@@ -376,12 +393,12 @@ class FunctionGradientsTest(test_util.TensorFlowTestCase):
   def testFunctionGradientWithGradFuncAndRegistration(self):
     g = ops.Graph()
     with g.as_default():
-      grad_func = function.Defun(tf.float32, tf.float32, tf.float32)(
-          self.XSquarePlusBGradient)
+      grad_func = function.Defun(dtypes.float32, dtypes.float32,
+                                 dtypes.float32)(self.XSquarePlusBGradient)
       with self.assertRaisesRegexp(ValueError, "Gradient defined twice"):
-        f = self._GetFunc(grad_func=grad_func,
-                          python_grad_func=self._PythonGradient)
-        f.add_to_graph(tf.Graph())
+        f = self._GetFunc(
+            grad_func=grad_func, python_grad_func=self._PythonGradient)
+        f.add_to_graph(ops.Graph())
 
 
 class StopGradientTest(test_util.TensorFlowTestCase):
@@ -394,6 +411,16 @@ class StopGradientTest(test_util.TensorFlowTestCase):
     assert igrad is None
 
 
+class PreventGradientTest(test_util.TensorFlowTestCase):
+
+  def testPreventGradient(self):
+    with ops.Graph().as_default():
+      inp = constant(1.0, shape=[100, 32], name="in")
+      out = array_ops.prevent_gradient(inp)
+      with self.assertRaisesRegexp(LookupError, "No gradient defined"):
+        _ = gradients.gradients(out, inp)
+
+
 class HessianVectorProductTest(test_util.TensorFlowTestCase):
 
   def testHessianVectorProduct(self):
@@ -426,7 +453,7 @@ class HessianTest(test_util.TensorFlowTestCase):
 
   def testHessian1D(self):
     # Manually compute the Hessian explicitly for a low-dimensional problem
-    # and check that `hessian` matches. Specifically, the Hessian of 
+    # and check that `hessian` matches. Specifically, the Hessian of
     # f(x) = x^T A x is H = A + A^T.
     m = 4
     rng = np.random.RandomState([1, 2, 3])
@@ -452,8 +479,10 @@ class HessianTest(test_util.TensorFlowTestCase):
     with self.test_session(use_gpu=True):
       mats = [constant_op.constant(mat_value) for mat_value in mat_values]
       xs = [constant_op.constant(x_value) for x_value in x_values]
-      xs_mats_xs = [math_ops.reduce_sum(x[:, None] * mat * x[None, :])
-                    for x, mat in zip(xs, mats)]
+      xs_mats_xs = [
+          math_ops.reduce_sum(x[:, None] * mat * x[None, :])
+          for x, mat in zip(xs, mats)
+      ]
       hessians = gradients.hessians(xs_mats_xs, xs)
       hessians_actual = [hess.eval() for hess in hessians]
     for hess_value, hess_actual in zip(hess_values, hessians_actual):
@@ -462,7 +491,7 @@ class HessianTest(test_util.TensorFlowTestCase):
   def testHessianInvalidDimension(self):
     for shape in [(10, 10), None]:
       with self.test_session(use_gpu=True):
-        x = array_ops.placeholder(tf.float32, shape)
+        x = array_ops.placeholder(dtypes.float32, shape)
         # Expect a ValueError because the dimensions are wrong
         with self.assertRaises(ValueError):
           gradients.hessians(x, x)
@@ -476,7 +505,7 @@ class IndexedSlicesToTensorTest(test_util.TensorFlowTestCase):
       c = constant_op.constant(np_val)
       c_sparse = math_ops._as_indexed_slices(c)
       self.assertAllEqual(np_val.shape, c_sparse.dense_shape.eval())
-      c_dense = math_ops.mul(c_sparse, 1.0)
+      c_dense = math_ops.multiply(c_sparse, 1.0)
       self.assertAllClose(np_val, c_dense.eval())
 
   def testIndexedSlicesToTensorList(self):
@@ -491,8 +520,8 @@ class IndexedSlicesToTensorTest(test_util.TensorFlowTestCase):
         numpy_list.append(np_val)
         dense_list.append(c)
         sparse_list.append(c_sparse)
-      packed_dense = array_ops.pack(dense_list)
-      packed_sparse = array_ops.pack(sparse_list)
+      packed_dense = array_ops.stack(dense_list)
+      packed_sparse = array_ops.stack(sparse_list)
       self.assertAllClose(packed_dense.eval(), packed_sparse.eval())
 
   def testInt64Indices(self):
@@ -501,42 +530,43 @@ class IndexedSlicesToTensorTest(test_util.TensorFlowTestCase):
       c = constant_op.constant(np_val)
       c_sparse = math_ops._as_indexed_slices(c)
       c_sparse = ops.IndexedSlices(
-          c_sparse.values, math_ops.cast(c_sparse.indices, dtypes.int64),
-          c_sparse.dense_shape)
+          c_sparse.values,
+          math_ops.cast(c_sparse.indices, dtypes.int64), c_sparse.dense_shape)
       self.assertAllEqual(np_val.shape, c_sparse.dense_shape.eval())
-      c_dense = math_ops.mul(c_sparse, 1.0)
+      c_dense = math_ops.multiply(c_sparse, 1.0)
       self.assertAllClose(np_val, c_dense.eval())
 
   def testWarnings(self):
     # Smaller than the threshold: no warning.
-    c_sparse = ops.IndexedSlices(array_ops.placeholder(dtypes.float32),
-                                 array_ops.placeholder(dtypes.int32),
-                                 constant([4, 4, 4, 4]))
+    c_sparse = ops.IndexedSlices(
+        array_ops.placeholder(dtypes.float32),
+        array_ops.placeholder(dtypes.int32), constant([4, 4, 4, 4]))
     with warnings.catch_warnings(record=True) as w:
-      math_ops.mul(c_sparse, 1.0)
+      math_ops.multiply(c_sparse, 1.0)
     self.assertEqual(0, len(w))
 
     # Greater than or equal to the threshold: warning.
-    c_sparse = ops.IndexedSlices(array_ops.placeholder(dtypes.float32),
-                                 array_ops.placeholder(dtypes.int32),
-                                 constant([100, 100, 100, 100]))
+    c_sparse = ops.IndexedSlices(
+        array_ops.placeholder(dtypes.float32),
+        array_ops.placeholder(dtypes.int32), constant([100, 100, 100, 100]))
     with warnings.catch_warnings(record=True) as w:
-      math_ops.mul(c_sparse, 1.0)
+      math_ops.multiply(c_sparse, 1.0)
     self.assertEqual(1, len(w))
     self.assertTrue(
-        "with 100000000 elements. This may consume a large amount of memory."
-        in str(w[0].message))
+        "with 100000000 elements. This may consume a large amount of memory." in
+        str(w[0].message))
 
     # Unknown dense shape: warning.
-    c_sparse = ops.IndexedSlices(array_ops.placeholder(dtypes.float32),
-                                 array_ops.placeholder(dtypes.int32),
-                                 array_ops.placeholder(dtypes.int32))
+    c_sparse = ops.IndexedSlices(
+        array_ops.placeholder(dtypes.float32),
+        array_ops.placeholder(dtypes.int32),
+        array_ops.placeholder(dtypes.int32))
     with warnings.catch_warnings(record=True) as w:
-      math_ops.mul(c_sparse, 1.0)
+      math_ops.multiply(c_sparse, 1.0)
     self.assertEqual(1, len(w))
     self.assertTrue(
-        "of unknown shape. This may consume a large amount of memory."
-        in str(w[0].message))
+        "of unknown shape. This may consume a large amount of memory." in
+        str(w[0].message))
 
 
 if __name__ == "__main__":
