@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
 """TensorArray operations.
 
 ## Classes containing dynamically sized arrays of Tensors.
@@ -25,8 +24,6 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from tensorflow.python.framework import constant_op
-from tensorflow.python.framework import dtypes as _dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.framework import tensor_util
@@ -35,6 +32,24 @@ from tensorflow.python.ops import gen_data_flow_ops
 from tensorflow.python.ops import math_ops
 
 
+def _maybe_set_device(handle_op, value_t):
+  # NOTE(ebrevdo): Do not try this at home, kids
+  # _______________________________________________
+  # | I WILL NOT ACCESS PRIVATE METHODS ^^^^^^^^\ |
+  # | I WILL NOT ACCESS PRIVATE METHODS |       | |
+  # | I WILL NOT ACCESS PRIVATE METHODS |_ __   | |
+  # | I WILL NOT ACCESS PRIVATE METHODS (.(. )  | |
+  # | I WILL NOT ACCESS PRIVATE         (_      ) |
+  # |                           \\      /___/' /  |
+  # |                           _\\_      \    |  |
+  # |                          ((   )     /====|  |
+  # |                           \  <.__._-      \ |
+  # |___________________________ <//___.         ||
+  #
+  if not handle_op.device and value_t.device:
+    handle_op._set_device(value_t.device)  # pylint: disable=protected-access
+
+
 # TensorArray object accesses many of the hidden generated ops, but is
 # in fact built to wrap these methods.
 # pylint: disable=protected-access
@@ -47,23 +62,34 @@ class TensorArray(object):
 
   @@handle
   @@flow
+  @@dtype
 
   @@read
   @@gather
-  @@pack
+  @@stack
   @@concat
 
   @@write
   @@scatter
-  @@unpack
+  @@unstack
   @@split
 
+  @@identity
+
   @@grad
   """
 
-  def __init__(self, dtype, size=None, dynamic_size=None,
-               clear_after_read=None, tensor_array_name=None, handle=None,
-               flow=None, infer_shape=True, name=None):
+  def __init__(self,
+               dtype,
+               size=None,
+               dynamic_size=None,
+               clear_after_read=None,
+               tensor_array_name=None,
+               handle=None,
+               flow=None,
+               infer_shape=True,
+               element_shape=None,
+               name=None):
     """Construct a new TensorArray or wrap an existing TensorArray handle.
 
     A note about the parameter `name`:
@@ -91,6 +117,9 @@ class TensorArray(object):
         `TensorArray.flow`.
       infer_shape: (optional, default: True) If True, shape inference
         is enabled.  In this case, all elements must have the same shape.
+      element_shape: (optional, default: None) A `TensorShape` object specifying
+        the shape constraints of each of the elements of the TensorArray.
+        Need not be fully defined.
       name: A name for the operation (optional).
 
     Raises:
@@ -107,6 +136,9 @@ class TensorArray(object):
     if handle is not None and size is not None:
       raise ValueError("Cannot provide both a handle and size "
                        "at the same time")
+    if handle is not None and element_shape is not None:
+      raise ValueError("Cannot provide both a handle and element_shape "
+                       "at the same time")
     if handle is not None and dynamic_size is not None:
       raise ValueError("Cannot provide both a handle and dynamic_size "
                        "at the same time")
@@ -119,31 +151,35 @@ class TensorArray(object):
     dynamic_size = dynamic_size or False
 
     self._dtype = dtype
-    self._infer_shape = infer_shape
-    # Record the current static shape for the array elements. The first
-    # write adds the shape of the tensor it writes, and all subsequent
-    # writes checks for shape equality.
-    self._elem_shape = []
+    # Record the current static shape for the array elements. The element
+    # shape is defined either by `element_shape` or the shape of the tensor
+    # of the first write. If `infer_shape` is true, all writes checks for
+    # shape equality.
+    if element_shape is None:
+      self._infer_shape = infer_shape
+      self._element_shape = []
+    else:
+      self._infer_shape = True
+      self._element_shape = [tensor_shape.TensorShape(element_shape)]
     with ops.name_scope(name, "TensorArray", [handle, size, flow]) as scope:
       if handle is not None:
         self._handle = handle
-      else:
-        if flow is not None:
-          with ops.colocate_with(flow):
-            self._handle = gen_data_flow_ops._tensor_array_v2(
-                dtype=dtype, size=size, dynamic_size=dynamic_size,
-                clear_after_read=clear_after_read,
-                tensor_array_name=tensor_array_name, name=scope)
-        else:
-          self._handle = gen_data_flow_ops._tensor_array_v2(
-              dtype=dtype, size=size, dynamic_size=dynamic_size,
-              clear_after_read=clear_after_read,
-              tensor_array_name=tensor_array_name, name=scope)
-      if flow is not None:
+        if flow is None:
+          raise ValueError("flow must not be None if handle is not None.")
         self._flow = flow
       else:
-        with ops.colocate_with(self._handle):
-          self._flow = constant_op.constant(0, dtype=_dtypes.float32)
+        # Construct the TensorArray with an empty device.  The first
+        # write into the TensorArray from a Tensor with a set device
+        # will retroactively set the device value of this op.
+        with ops.device(None), ops.colocate_with(None, ignore_existing=True):
+          self._handle, self._flow = gen_data_flow_ops._tensor_array_v3(
+              dtype=dtype,
+              size=size,
+              element_shape=element_shape,
+              dynamic_size=dynamic_size,
+              clear_after_read=clear_after_read,
+              tensor_array_name=tensor_array_name,
+              name=scope)
 
   @property
   def flow(self):
@@ -160,6 +196,40 @@ class TensorArray(object):
     """The reference to the TensorArray."""
     return self._handle
 
+  def _merge_element_shape(self, shape):
+    """Changes the element shape of the array given a shape to merge with.
+
+    Args:
+      shape: A `TensorShape` object to merge with.
+
+    Raises:
+      ValueError: if the provided shape is incompatible with the current
+          element shape of the `TensorArray`.
+    """
+
+    if self._element_shape:
+      if not shape.is_compatible_with(self._element_shape[0]):
+        raise ValueError(
+            "Inconsistent shapes: saw %s but expected %s "
+            "(and infer_shape=True)" % (shape, self._element_shape[0]))
+      self._element_shape[0] = self._element_shape[0].merge_with(shape)
+    else:
+      self._element_shape.append(shape)
+
+  def identity(self):
+    """Returns a TensorArray with the same content and properties.
+
+    Returns:
+      A new TensorArray object with flow that ensures the control dependencies
+      from the contexts will become control dependencies for writes, reads, etc.
+      Use this object all for subsequent operations.
+    """
+    flow = array_ops.identity(self._flow)
+    ta = TensorArray(dtype=self._dtype, handle=self._handle, flow=flow,
+                     infer_shape=self._infer_shape)
+    ta._element_shape = self._element_shape
+    return ta
+
   def grad(self, source, flow=None, name=None):
     # tensor_array_grad requires a flow input when forward
     # TensorArrays are dynamically sized.  This forces the creation
@@ -169,12 +239,16 @@ class TensorArray(object):
       flow = self.flow
     with ops.name_scope(name, "TensorArrayGrad", [self._handle]):
       with ops.colocate_with(self._handle):
-        g_handle = gen_data_flow_ops._tensor_array_grad_v2(
+        g_handle, unused_flow = gen_data_flow_ops._tensor_array_grad_v3(
             handle=self._handle, source=source, flow_in=flow, name=name)
         with ops.control_dependencies([g_handle]):
           flow = array_ops.identity(flow, name="gradient_flow")
-        g = TensorArray(dtype=self._dtype, handle=g_handle, flow=flow,
-                        infer_shape=self._infer_shape)
+        g = TensorArray(
+            dtype=self._dtype,
+            handle=g_handle,
+            flow=flow,
+            infer_shape=self._infer_shape)
+        g._element_shape = self._element_shape
         return g
 
   def read(self, index, name=None):
@@ -188,11 +262,14 @@ class TensorArray(object):
       The tensor at index `index`.
     """
     with ops.colocate_with(self._handle):
-      value = gen_data_flow_ops._tensor_array_read_v2(
-          handle=self._handle, index=index, flow_in=self._flow,
-          dtype=self._dtype, name=name)
-      if self._elem_shape:
-        value.set_shape(self._elem_shape[0].dims)
+      value = gen_data_flow_ops._tensor_array_read_v3(
+          handle=self._handle,
+          index=index,
+          flow_in=self._flow,
+          dtype=self._dtype,
+          name=name)
+      if self._element_shape:
+        value.set_shape(self._element_shape[0].dims)
       return value
 
   def write(self, index, value, name=None):
@@ -210,38 +287,37 @@ class TensorArray(object):
     Raises:
       ValueError: if there are more writers than specified.
     """
-    with ops.colocate_with(self._handle):
-      flow_out = gen_data_flow_ops._tensor_array_write_v2(
-          handle=self._handle, index=index, value=value, flow_in=self._flow,
-          name=name)
-      ta = TensorArray(dtype=self._dtype, handle=self._handle)
-      ta._flow = flow_out
+    with ops.name_scope(name, "TensorArrayWrite", [self._handle, index, value]):
+      value = ops.convert_to_tensor(value, name="value")
+      _maybe_set_device(self._handle.op, value)
+      with ops.colocate_with(self._handle):
+        flow_out = gen_data_flow_ops._tensor_array_write_v3(
+            handle=self._handle,
+            index=index,
+            value=value,
+            flow_in=self._flow,
+            name=name)
+      ta = TensorArray(dtype=self._dtype, handle=self._handle, flow=flow_out)
       ta._infer_shape = self._infer_shape
-      ta._elem_shape = self._elem_shape
+      ta._element_shape = self._element_shape
       if ta._infer_shape:
-        val_shape = flow_out.op.inputs[2].get_shape()
-        if ta._elem_shape:
-          if not val_shape == ta._elem_shape[0]:
-            raise ValueError(
-                "Inconsistent shapes: saw %s but expected %s "
-                "(and infer_shape=True)" % (val_shape, ta._elem_shape[0]))
-        else:
-          ta._elem_shape.append(val_shape)
+        ta._merge_element_shape(value.get_shape())
       return ta
 
-  def pack(self, name=None):
-    """Return the values in the TensorArray as a packed `Tensor`.
+  def stack(self, name=None):
+    """Return the values in the TensorArray as a stacked `Tensor`.
 
     All of the values must have been written and their shapes must all match.
+    If input shapes have rank-`R`, then output shape will have rank-`(R+1)`.
 
     Args:
       name: A name for the operation (optional).
 
     Returns:
-      All the tensors in the TensorArray packed into one tensor.
+      All the tensors in the TensorArray stacked into one tensor.
     """
     with ops.colocate_with(self._handle):
-      with ops.name_scope(name, "TensorArrayPack", [self._handle]):
+      with ops.name_scope(name, "TensorArrayStack", [self._handle]):
         return self.gather(math_ops.range(0, self.size()), name=name)
 
   def gather(self, indices, name=None):
@@ -259,19 +335,19 @@ class TensorArray(object):
       The in the `TensorArray` selected by `indices`, packed into one tensor.
     """
     with ops.colocate_with(self._handle):
-      if self._elem_shape:
-        element_shape = self._elem_shape[0]
+      if self._element_shape:
+        element_shape = self._element_shape[0]
       else:
         element_shape = tensor_shape.TensorShape(None)
-      value = gen_data_flow_ops._tensor_array_gather_v2(
+      value = gen_data_flow_ops._tensor_array_gather_v3(
           handle=self._handle,
           indices=indices,
           flow_in=self._flow,
           dtype=self._dtype,
           name=name,
           element_shape=element_shape)
-      if self._elem_shape and self._elem_shape[0].dims is not None:
-        value.set_shape([None] + self._elem_shape[0].dims)
+      if self._element_shape and self._element_shape[0].dims is not None:
+        value.set_shape([None] + self._element_shape[0].dims)
       return value
 
   def concat(self, name=None):
@@ -286,41 +362,42 @@ class TensorArray(object):
     Returns:
       All the tensors in the TensorArray concatenated into one tensor.
     """
-    if self._elem_shape and self._elem_shape[0].dims is not None:
-      element_shape_except0 = tensor_shape.TensorShape(self._elem_shape[0].dims[
-          1:])
+    if self._element_shape and self._element_shape[0].dims is not None:
+      element_shape_except0 = (
+          tensor_shape.TensorShape(self._element_shape[0].dims[1:]))
     else:
       element_shape_except0 = tensor_shape.TensorShape(None)
     with ops.colocate_with(self._handle):
-      value, _ = gen_data_flow_ops._tensor_array_concat_v2(
+      value, _ = gen_data_flow_ops._tensor_array_concat_v3(
           handle=self._handle,
           flow_in=self._flow,
           dtype=self._dtype,
           name=name,
           element_shape_except0=element_shape_except0)
-      if self._elem_shape and self._elem_shape[0].dims is not None:
-        value.set_shape([None] + self._elem_shape[0].dims[1:])
+      if self._element_shape and self._element_shape[0].dims is not None:
+        value.set_shape([None] + self._element_shape[0].dims[1:])
       return value
 
-  def unpack(self, value, name=None):
-    """Pack the values of a `Tensor` in the TensorArray.
+  def unstack(self, value, name=None):
+    """Unstack the values of a `Tensor` in the TensorArray.
 
+    If input value shapes have rank-`R`, then the output TensorArray will
+    contain elements whose shapes are rank-`(R-1)`.
     Args:
-      value: (N+1)-D.  Tensor of type `dtype`.  The Tensor to unpack.
+      value: (N+1)-D.  Tensor of type `dtype`.  The Tensor to unstack.
       name: A name for the operation (optional).
 
     Returns:
-      A new TensorArray object with flow that ensures the unpack occurs.
+      A new TensorArray object with flow that ensures the unstack occurs.
       Use this object all for subsequent operations.
 
     Raises:
       ValueError: if the shape inference fails.
     """
-    with ops.colocate_with(self._handle):
-      with ops.name_scope(name, "TensorArrayPack", [self._handle, value]):
-        num_elements = array_ops.shape(value)[0]
-        return self.scatter(
-            indices=math_ops.range(0, num_elements), value=value, name=name)
+    with ops.name_scope(name, "TensorArrayUnstack", [self._handle, value]):
+      num_elements = array_ops.shape(value)[0]
+      return self.scatter(
+          indices=math_ops.range(0, num_elements), value=value, name=name)
 
   def scatter(self, indices, value, name=None):
     """Scatter the values of a `Tensor` in specific indices of a `TensorArray`.
@@ -338,26 +415,26 @@ class TensorArray(object):
     Raises:
       ValueError: if the shape inference fails.
     """
-    with ops.colocate_with(self._handle):
-      flow_out = gen_data_flow_ops._tensor_array_scatter_v2(
-          handle=self._handle, indices=indices, value=value, flow_in=self._flow,
-          name=name)
-      ta = TensorArray(dtype=self._dtype, handle=self._handle)
-      ta._flow = flow_out
+    with ops.name_scope(name, "TensorArrayScatter",
+                        [self._handle, value, indices]):
+      value = ops.convert_to_tensor(value, name="value")
+      _maybe_set_device(self._handle.op, value)
+      with ops.colocate_with(self._handle):
+        flow_out = gen_data_flow_ops._tensor_array_scatter_v3(
+            handle=self._handle,
+            indices=indices,
+            value=value,
+            flow_in=self._flow,
+            name=name)
+      ta = TensorArray(dtype=self._dtype, handle=self._handle, flow=flow_out)
       ta._infer_shape = self._infer_shape
-      ta._elem_shape = self._elem_shape
+      ta._element_shape = self._element_shape
       if ta._infer_shape:
         val_shape = flow_out.op.inputs[2].get_shape()
-        elem_shape = tensor_shape.unknown_shape()
+        element_shape = tensor_shape.unknown_shape()
         if val_shape.dims is not None:
-          elem_shape = tensor_shape.TensorShape(val_shape.dims[1:])
-        if ta._elem_shape:
-          if not elem_shape == ta._elem_shape[0]:
-            raise ValueError(
-                "Inconsistent shapes: saw %s but expected %s "
-                "(and infer_shape=True)" % (elem_shape, ta._elem_shape[0]))
-        else:
-          ta._elem_shape.append(elem_shape)
+          element_shape = tensor_shape.TensorShape(val_shape.dims[1:])
+        ta._merge_element_shape(element_shape)
       return ta
 
   def split(self, value, lengths, name=None):
@@ -376,44 +453,42 @@ class TensorArray(object):
     Raises:
       ValueError: if the shape inference fails.
     """
-    with ops.colocate_with(self._handle):
-      with ops.name_scope(name, "TensorArraySplit",
-                          [self._handle, value, lengths]):
-        lengths_64 = math_ops.to_int64(lengths)
-      flow_out = gen_data_flow_ops._tensor_array_split_v2(
-          handle=self._handle, value=value, lengths=lengths_64,
-          flow_in=self._flow, name=name)
-      ta = TensorArray(dtype=self._dtype, handle=self._handle)
-      ta._flow = flow_out
+    with ops.name_scope(name, "TensorArraySplit",
+                        [self._handle, value, lengths]):
+      value = ops.convert_to_tensor(value, name="value")
+      _maybe_set_device(self._handle.op, value)
+      lengths_64 = math_ops.to_int64(lengths)
+      with ops.colocate_with(self._handle):
+        flow_out = gen_data_flow_ops._tensor_array_split_v3(
+            handle=self._handle,
+            value=value,
+            lengths=lengths_64,
+            flow_in=self._flow,
+            name=name)
+      ta = TensorArray(dtype=self._dtype, handle=self._handle, flow=flow_out)
       ta._infer_shape = self._infer_shape
-      ta._elem_shape = self._elem_shape
+      ta._element_shape = self._element_shape
       if ta._infer_shape:
         val_shape = flow_out.op.inputs[1].get_shape()
         clengths = tensor_util.constant_value(flow_out.op.inputs[2])
-        elem_shape = tensor_shape.unknown_shape()
+        element_shape = tensor_shape.unknown_shape()
         if val_shape.dims is not None:
           if clengths is not None and clengths.max() == clengths.min():
-            elem_shape = tensor_shape.TensorShape(
-                [clengths[0]] + val_shape.dims[1:])
-        if ta._elem_shape:
-          if not elem_shape == ta._elem_shape[0]:
-            raise ValueError(
-                "Inconsistent shapes: saw %s but expected %s "
-                "(and infer_shape=True)" % (elem_shape, ta._elem_shape[0]))
-        else:
-          ta._elem_shape.append(elem_shape)
+            element_shape = tensor_shape.TensorShape([clengths[0]] +
+                                                     val_shape.dims[1:])
+        ta._merge_element_shape(element_shape)
       return ta
 
   def size(self, name=None):
     """Return the size of the TensorArray."""
     with ops.colocate_with(self._handle):
-      return gen_data_flow_ops._tensor_array_size_v2(
+      return gen_data_flow_ops._tensor_array_size_v3(
           handle=self._handle, flow_in=self.flow, name=name)
 
   def close(self, name=None):
     """Close the current TensorArray."""
     with ops.colocate_with(self._handle):
-      return gen_data_flow_ops._tensor_array_close_v2(
+      return gen_data_flow_ops._tensor_array_close_v3(
           handle=self._handle, name=name)
 
 # pylint: enable=protected-access
